<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2010 Glowbase GmbH <support@glowbase.com>
 *  All rights reserved
 *
 ***************************************************************/

namespace qfq;

//use qfq;

require_once(__DIR__ . '/Define.php');
require_once(__DIR__ . '/Utils.php');
require_once(__DIR__ . '/Variables.php');
require_once(__DIR__ . '/Error.php');
//require_once(__DIR__ . '/Db.php');
require_once(__DIR__ . '/../Database.php');
require_once(__DIR__ . '/Link.php');
require_once(__DIR__ . '/Sendmail.php');
require_once(__DIR__ . '/../exceptions/UserReportExtension.php');
require_once(__DIR__ . '/../Evaluate.php');


class Report {

    /**
     * @var SIP
     */
    private $sip = null;

    /**
     * @var Store
     */
    private $store = null;

    /**
     * @var string
     */
    private $dbAlias = '';
    /**
     * @var Log
     */
    private $log = null;

    // frArray[10.50.5.sql][select ...]
    private $frArray = array();

    // $indexArray[10][50][5]   one entry per 'sql' statement
    private $indexArray = array();

    // TODO to explain
//	private $resultArray = array();
    private $levelCount = 0;
    //private $counter = 0;
    /**
     * @var Variables
     */
    private $variables = null;

    /**
     * @var Utils
     */
    private $utils = null;

    /**
     * @var Db
     */
    private $db = null;

    /**
     * @var Sendmail
     */
    private $sendmail = null;

    private $page_control = array();

    // Emulate global variable: will be set much earlier in other functions. Will be shown in error messages.
    private $fr_error = array('uid' => '', 'pid' => '', 'row' => '', 'debug_level' => '0', 'full_level' => '');

    private $t3data = array();

    private $phpUnit = false;

    private $showDebugInfo = false;

    /**
     * Report constructor.
     *
     * @param array $t3data
     */
    public function __construct(array $t3data, $sessionName, Evaluate $eval, $phpUnit = false) {

        $this->phpUnit = $phpUnit;

        $this->t3data = $t3data;
        $this->sip = new Sip($sessionName, $phpUnit);
        $this->store = Store::getInstance();

        $this->showDebugInfo = ($this->store->getVar(SYSTEM_SHOW_DEBUG_INFO, STORE_SYSTEM) === 'yes');

        $this->page_control["msgbox"]["pagec"] = "Please confirm!";

        $this->page_control["hash"]["paged"] = "h";
        $this->page_control["hash"]["pagee"] = "h";
        $this->page_control["hash"]["pagen"] = "h";

        $this->page_control["icon"]["paged"] = "D";
        $this->page_control["icon"]["pagee"] = "E";
        $this->page_control["icon"]["pageh"] = "H";
        $this->page_control["icon"]["pagei"] = "I";
        $this->page_control["icon"]["pagen"] = "N";
        $this->page_control["icon"]["pages"] = "S";

        $this->db = new Database();
        $this->utils = new Utils();
        $this->variables = new Variables($eval, $t3data["uid"]);

        // Set static values, which won't change during this run.
        $this->fr_error["pid"] = isset($this->variables->resultArray['global.']['page_id']) ? $this->variables->resultArray['global.']['page_id'] : 0;
        $this->fr_error["uid"] = $t3data["uid"];
        $this->fr_error["debug_level"] = 0;

        // Sanitize function for POST and GET Parameters.
        // Merged URL-Parameter (key1, id etc...) in resultArray.
        $this->variables->resultArray = array_merge($this->variables->resultArray, array("global." => $this->variables->collectGlobalVariables()));

        // Create Logclass.
        $this->log = new Log($this->variables->resultArray['global.']);

        // Create sendmail Class. Take care to prepare a fr_log instance.
        $this->sendmail = new Sendmail($this->log);
    }

    /**
     * Main function. Parses bodytext and iterates over all queries.
     *
     * @return string
     */
    public function process() {

        $this->log->set_fr_error($this->fr_error);

        // Iteration over Bodytext
        $ttLineArray = explode("\n", $this->t3data['bodytext']);

        foreach ($ttLineArray as $index => $line) {
            // Fill $frArray, $indexArray, $resultArray
            $this->parseFRLine($line);
        }

        // Sort array
        $this->sortIndexArray($this->indexArray, $this->generateSortArg());

        // Report
        $content = $this->triggerReport();

        return $content;
    }

    /**
     * Split line in level, command, content and fill 'frArray', 'levelCount', 'indexArray'
     * Example: 10.50.5.sql = select * from person
     *
     * @param    string $ttLine : line to split in level, command, content
     * @return    void
     */
    private function parseFRLine($ttLine) {

        // 10.50.5.sql = select ...
        $arr = explode("=", trim($ttLine), 2);

        // 10.50.5.sql
        $key = strtolower(trim($arr[0]));

        // comment ?
        if (substr($key, 0, 1) == "#") return;

        // select ...
        $value = trim($arr[1]);

        // 10.50.5.sql
        $arrKey = explode('.', $key);

        // frCmd = "sql"
        $frCmd = $arrKey[count($arrKey) - 1];

        // remove last item (cmd)
        unset($arrKey[count($arrKey) - 1]);

        // save elements only if there is a level specified
        if (count($arrKey)) {
            // level = "10.50.5"
            $level = implode(".", $arrKey);

            // fill Array
            $this->setLine($level, $frCmd, $value);
        }
    }

    /**
     * @param $level
     * @param $frCmd
     * @param $value
     */
    private function setLine($level, $frCmd, $value) {

        // store complete line reformatted in frArray
        $this->frArray[$level . "." . $frCmd] = $value;

        // per sql command
        //pro sql cmd wir der Indexarray abgefüllt. Dieser wird später verwendet um auf den $frArray zuzugreifen
        //if(preg_match("/^sql/i", $frCmd) == 1){
        if ($frCmd == "sql" || $frCmd == "form") {
            // Remember max level
            $this->levelCount = max(substr_count($level, '.') + 1, $this->levelCount);
            // $indexArray[10][50][5]
            $this->indexArray[] = explode(".", $level);
        }
    }

    /**
     * Sorts the associative array.
     *
     * @param    array $ary : The unsorted Level Array
     * @param    string $clause : the sort argument 0 ASC, 1 ASC... according to the number of columns
     * @return    The content that is displayed on the website
     */
    private function sortIndexArray(array &$ary, $clause, $ascending = true) {

        $clause = str_ireplace('order by', '', $clause);
        $clause = preg_replace('/\s+/', ' ', $clause);
        $keys = explode(',', $clause);
        $dirMap = array('desc' => 1, 'asc' => -1);
        $def = $ascending ? -1 : 1;

        $keyAry = array();
        $dirAry = array();
        foreach ($keys as $key) {
            $key = explode(' ', trim($key));
            $keyAry[] = trim($key[0]);
            if (isset($key[1])) {
                $dir = strtolower(trim($key[1]));
                $dirAry[] = $dirMap[$dir] ? $dirMap[$dir] : $def;
            } else {
                $dirAry[] = $def;
            }
        }
        $fnBody = '';
        for ($i = count($keyAry) - 1; $i >= 0; $i--) {
            $k = $keyAry[$i];
            $t = $dirAry[$i];
            $f = -1 * $t;
            $aStr = '$a[\'' . $k . '\']';
            $bStr = '$b[\'' . $k . '\']';

            if (strpos($k, '(') !== false) {
                $aStr = '$a->' . $k;
                $bStr = '$b->' . $k;
            }

            if ($fnBody == '') {
                $fnBody .= "if({$aStr} == {$bStr}) { return 0; }\n";
                $fnBody .= "return ({$aStr} < {$bStr}) ? {$t} : {$f};\n";
            } else {
                $fnBody = "if({$aStr} == {$bStr}) {\n" . $fnBody;
                $fnBody .= "}\n";
                $fnBody .= "return ({$aStr} < {$bStr}) ? {$t} : {$f};\n";
            }
        }

        if ($fnBody) {
            $sortFn = create_function('$a,$b', $fnBody);
            usort($ary, $sortFn);
        }
    }

    /**
     * generateSortArg
     *
     * @return string
     */
    private function generateSortArg() {

        $sortArg = "";

        for ($i = 0; $i < $this->levelCount; $i++) {
            $sortArg = $sortArg . $i . " ASC, ";
        }
        $sortArg = substr($sortArg, 0, strlen($sortArg) - 2);
        return $sortArg;
    }

    /**
     * Executes the queries recursive. This Method is called for each Sublevel.
     *
     * ROOTLEVEL
     * This method is called once from the main method.
     * For the first call the method executes the rootlevels
     *
     * SUBLEVEL
     * For each rootlevel the method calls it self whith the levelmode 0
     * If the next Level is a Sublevel it will be executed and $this->counter will be added by 1
     * The sublevel calls the method again for a following sublevel
     *
     * @param int $cur_level Which level it will call [10] = level 1, [10.10] = level 2 ...
     * @param string $super_level_array The Value-Array of the indexarray [0=>10, 1=>50]
     * @param int $counter The outer numeric Arraykey of indexarray
     * @return string                       The content that is displayed on the website
     * @throws codeException
     * @throws SqlReportException
     * @throws SyntaxReportException
     * @throws UserReportException
     */

    private function triggerReport($cur_level = 1, array $super_level_array = array(), $counter = 0) {
        $keys = array();
        $stat = array();

        $lineDebug = 0;
        $content = "";
        $rowTotal = 0;

        // CurrentLevel "10.10.50"
        if (isset($this->indexArray[$counter]) && is_array($this->indexArray[$counter])) {
            $full_level = implode(".", $this->indexArray[$counter]);
        } else {
            $full_level = '';
        }

        // Superlevel "10.10"
        if (isset($super_level_array) && is_array($super_level_array)) {
            $full_super_level = implode(".", $super_level_array);
        } else {
            $full_super_level = '';
        }

        //condition1: indexArray
        //condition2: full_level == Superlevel (but at the length of the Superlevel)
        while ($counter < count($this->indexArray) && $full_super_level == substr($full_level, 0, strlen($full_super_level))) {

            //True: The cur_level is a subquery -> continue
            if ($cur_level != count($this->indexArray[$counter])) {
                ++$counter;
                if (isset($this->indexArray[$counter]) && is_array($this->indexArray[$counter])) {
                    $full_level = implode(".", $this->indexArray[$counter]);
                } else {
                    $full_level = '';
                }
                continue;
            }

            // Set dbAlias if one is specified. Else keep the parent one.
            //TODO dbAlias
//            $this->dbAlias = $this->getValueParentDefault("db", $full_super_level, $full_level, $cur_level, DB);

            // Set debug, if one is specified else keep the parent one.
            $lineDebug = $this->getValueParentDefault("debug", $full_super_level, $full_level, $cur_level, 0);

            // Prepare Error reporting
            $this->store->setVar(SYSTEM_SQL_RAW, $this->frArray[$full_level . ".sql"], STORE_SYSTEM);
            $this->store->setVar(SYSTEM_REPORT_FULL_LEVEL, $full_level, STORE_SYSTEM);

            // Prepare SQL: replace variables. Actual 'line.total' or 'line.count' will recalculated: don't replace them now!
            unset($this->variables->resultArray[$full_level . ".line."]["total"]);
            unset($this->variables->resultArray[$full_level . ".line."]["count"]);
            $sql = $this->variables->doVariables($this->frArray[$full_level . ".sql"]);

            $this->store->setVar(SYSTEM_SQL_FINAL, $sql, STORE_SYSTEM);

            //Execute SQL. All errors have been already catched.
            unset($result);
            $result = $this->db->sqlKeys($sql, $keys, $stat);

            // If an array is returned, $sql was a query, otherwise an 'insert', 'update', 'delete', ...
            // Query: total nummber of rows
            // insert, delete, update: number of affected rows
            $rowTotal = isset($stat[DB_NUM_ROWS]) ? $stat[DB_NUM_ROWS] : $stat[DB_AFFECTED_ROWS];

            $this->variables->resultArray[$full_level . ".line."]["total"] = $rowTotal;
            if (isset($stat[DB_INSERT_ID])) {
                $this->variables->resultArray[$full_level . ".line."]["insertId"] = $stat[DB_INSERT_ID];
            }

            // HEAD: If there is at least one record, do 'head'.
            if ($rowTotal > 0)
                $content .= $this->variables->doVariables($this->frArray[$full_level . "." . "head"]);

            // Prepare row alteration
            $arrRbgd = explode("|", $this->frArray[$full_level . "." . "rbgd"]);

            if (is_array($result)) {
                //---------------------------------
                // Process each row of resultset
                $columnValueSeperator = "";
                $rowIndex = 0;
                foreach ($result as $row) {
                    // record number counter
                    $this->variables->resultArray[$full_level . ".line."]["count"] = ++$rowIndex;

                    // replace {{<level>.line.count}} and {{<level>.line.total}} in __result__, if the variables specify their own full_level. This can't be replaced before firing the query.
                    for ($ii = 0; $ii < count($row); $ii++) {
                        $row[$ii] = str_replace("{{" . $full_level . ".line.count}}", $rowIndex, $row[$ii]);
                        $row[$ii] = str_replace("{{" . $full_level . ".line.total}}", $rowTotal, $row[$ii]);
                    }

                    // SEP set seperator (empty on first run)
                    $content .= $columnValueSeperator;
                    $columnValueSeperator = $this->variables->doVariables($this->frArray[$full_level . "." . "rsep"]);

                    // RBGD: even/odd rows
                    $content .= str_replace("rbgd", $arrRbgd[$rowIndex % 2], $this->frArray[$full_level . "." . "rbeg"]);

                    //-----------------------------
                    // COLUMNS: Collect all columns
                    $content .= $this->collectRow($row, $keys, $full_level, $rowIndex);

                    // REND
                    $content .= $this->variables->doVariables($this->frArray[$full_level . "." . "rend"]);

                    // Trigger subqueries of this level
                    $content .= $this->triggerReport($cur_level + 1, $this->indexArray[$counter], $counter + 1);

                    // RENR
                    $content .= $this->variables->doVariables($this->frArray[$full_level . "." . "renr"]);

                }
            }

            //Print althead or tail
            if ($rowTotal > 0) {
                $content .= $this->variables->doVariables($this->frArray[$full_level . "." . "tail"]);
            } else {
                $content .= $this->variables->doVariables($this->frArray[$full_level . "." . "althead"]);
            }

            ++$counter;
            if (isset($this->indexArray[$counter]) && is_array($this->indexArray[$counter])) {
                $full_level = implode(".", $this->indexArray[$counter]);
            } else {
                $full_level = '';
            }
        }

        return $content;
    }

    /**
     * Determine value:
     * 1) if one specified in line: take it
     * 2) if one specified in upper level: take it
     * 3) if none above take default
     * Set value on $full_level
     *
     * @param    string $level_key - 'db' or 'debug'
     * @param    string $full_super_level - f.e.: 10.10.
     * @param    string $full_level - f.e.: 10.10.10.
     * @param    string $cur_level - f.e.: 2
     * @param    string $default - f.e.: 0
     * @return   string  The calculated value.
     */
    private function getValueParentDefault($level_key, $full_super_level, $full_level, $cur_level, $default) {

        if ($this->frArray[$full_level . "." . $level_key]) {
            $value = $this->frArray[$full_level . "." . $level_key];
        } else {
            if ($cur_level == 1) {
                $value = $default;
            } else {
                $value = $this->variables->resultArray[$full_super_level . ".line."][$level_key];
            }
        }
        $this->variables->resultArray[$full_level . ".line."][$level_key] = $value;

        return ($value);
    }

    /**
     * Steps through 'row' and collects all columns
     *
     * @param array $row Recent row fetch from sql resultset.
     * @param array $keys List of all columnnames
     * @param string $full_level Recent position to work on.
     * @param string $rowIndex Index of recent row in resultset.
     * @return string               Collected content of all printable columns
     * @throws SyntaxReportException
     */
    private function collectRow(array $row, array $keys, $full_level, $rowIndex) {
        $content = "";

        $fsep = '';
        for ($ii = 0; $ii < count($keys); $ii++) {

            $this->store->setVar(SYSTEM_REPORT_COLUMN_INDEX, $ii + 1, STORE_SYSTEM);
            $this->store->setVar(SYSTEM_REPORT_COLUMN_NAME, $keys[$ii], STORE_SYSTEM);
            $this->store->setVar(SYSTEM_REPORT_COLUMN_VALUE, $row[$ii], STORE_SYSTEM);

            $flagOutput = false;
            $renderedColumn = $this->renderColumn($ii, $keys[$ii], $row[$ii], $full_level, $rowIndex, $flagOutput);
            if ($flagOutput) {
                //prints
                $content .= $this->variables->doVariables($fsep);
                $content .= $this->variables->doVariables($this->frArray[$full_level . "." . "fbeg"]);
                $content .= $renderedColumn;
                $content .= $this->variables->doVariables($this->frArray[$full_level . "." . "fend"]);
                $fsep = $this->frArray[$full_level . "." . "fsep"];
            }
        }
        return ($content);
    }

    /**
     * Renders column depending of column name (if name is a reserved column name)
     *
     * @param string $columnIndex
     * @param string $columnName
     * @param string $columnValue
     * @param string $full_level
     * @param string $rowIndex
     * @return string rendered column
     * @throws SyntaxReportException
     */
    private function renderColumn($columnIndex, $columnName, $columnValue, $full_level, $rowIndex, &$flagOutput) {
        $content = "";
        $flagControl = false;
        $flagOutput = true;

        if (substr($columnName, 0, 1) == "_") {
            $flagControl = true;
            $columnName = substr($columnName, 1);
        }
        //TODO: reserved names,not starting with '_' will be still accepted - stop this!
        switch ($columnName) {
            case "link":
                $link = new Link($this->fr_error, $this->sip);
                $content .= $link->renderLink($columnValue);
#				unset $link;
                break;
            case "exec":
                $content .= $this->myExec($columnValue);
                break;

            case "Page":
            case "Pagec":
            case "Paged":
            case "Pagee":
            case "Pageh":
            case "Pagei":
            case "Pagen":
            case "Pages":
                $linkValue = $this->doFixColPosPage($columnName, $columnValue);

            $link = new Link($this->fr_error, $this->sip);
            $content .= $link->renderLink($linkValue);
                break;

            case "page":
            case "pagec":
            case "paged":
            case "pagee":
            case "pageh":
            case "pagei":
            case "pagen":
            case "pages":
#debug($columnValue);
                $linkValue = $this->doPage($columnName, $columnValue);
// debug($linkValue);

            $link = new Link($this->fr_error, $this->sip);
            $content .= $link->renderLink($linkValue);
                break;

            case "bullet":
                if ($columnValue === '')
                    break;

                $linkValue = "r:3|B:" . $columnValue;
                $link = new Link($this->fr_error, $this->sip);
                $content .= $link->renderLink($linkValue);
                break;

            case "check":
                if ($columnValue === '')
                    break;

                $linkValue = "r:3|C:" . $columnValue;
                $link = new Link($this->fr_error, $this->sip);
                $content .= $link->renderLink($linkValue);
                break;

            case "img":
                // "<path to image>|[alttext]|[text behind]" renders to: <img src="<path to image>" alt="[alttext]">[text behind]
                if (empty($columnValue)) break;
                $tmp = explode("|", $columnValue, 3);
                if ($tmp[0] == "") break;
                $content .= '<img src="' . $tmp[0] . '" alt="' . $tmp[1] . '">' . $tmp[2];
                break;

            case "mailto":
                // "<email address>|[Real Name]"  renders to (encrypted via JS): <a href="mailto://<email address>"><email address></a> OR <a href="mailto://<email address>">[Real Name]</a>
                $tmp = explode("|", $columnValue, 2);
                if ($tmp[0] == "") break;

                $t1 = explode("@", $tmp[0], 2);
                $content .= "<script language=javascript><!--" . chr(10);
                if (empty($tmp[1])) $tmp[1] = $tmp[0];

                $content .= 'var contact = "' . substr($tmp[1], 0, 2) . '"' . chr(10);
                $content .= 'var contact1 = "' . substr($tmp[1], 2) . '"' . chr(10);
                $content .= 'var email = "' . $t1[0] . '"' . chr(10);
                $content .= 'var emailHost = "' . $t1[1] . '"' . chr(10);

                $content .= 'document.write("<a href=" + "mail" + "to:" + email + "@" + emailHost+ ">" + contact + "</a>")' . chr(10);
                $content .= 'document.write("<a href=" + "mail" + "to:" + email + "@" + emailHost+ ">" + contact1 + "</a>")' . chr(10);
                $content .= '//--></script>';
                break;

            case "sendmail":
                // 'Absender|Empfaenger, mehrere mit Komma getrennt|Betreff|Mailinhalt'
                $tmp = explode("|", $columnValue, 4);

                $mail['receiver'] = $tmp[0];
                $mail['sender'] = $tmp[1];
                $mail['subject'] = $tmp[2];
                $mail['body'] = $tmp[3];

                $content = $this->sendmail->sendmail($mail);
                break;

            case "vertical":
                // '<Text>|[angle]|[width]|[height]|[tag]'   , width and heigth needs a unit (px, em ,...), 'tag' might be 'div', 'span', ...
                $arr = explode("|", $columnValue, 5);

                # angle
                $angle = $arr[1] ? $arr[1] : 270;

                # width
                $width = $arr[2] ? $arr[2] : "1em";
                $tmp = "width:$width; ";

                # height
                if ($arr[3])
                    $tmp .= "height:" . $arr[3] . "; ";

                # tag
                if ($arr[4]) {
                    $tag = explode(" ", trim($arr[4]), 2);
                    $tag_open = "<" . $arr[4] . " ";
                    $tag_close = "</" . $tag[0] . ">";
                } else {
                    $tag_open = "<div ";
                    $tag_close = "</div>";
                }

                # http://scottgale.com/blog/css-vertical-text/2010/03/01/
                #$style = "writing-mode:tb-rl; -webkit-transform:rotate(270deg); -moz-transform:rotate(270deg); -o-transform: rotate(270deg); white-space:nowrap; width:1em;  display: block;";
                $style = "width:1em;  filter: flipv fliph; transform: rotate(" . $angle . "deg) translate(-10em,0); transform-origin: 0 0; writing-mode:tb-rl; -webkit-transform:rotate(" . $angle . "deg); -moz-transform:rotate(" . $angle . "deg); -o-transform: rotate(" . $angle . "deg); white-space:nowrap; display: block;";

                #$style = "line-height: 1.5em; background:#eee; display: block; white-space: nowrap; padding-left: 3px; writing-mode: tb-rl; filter: flipv fliph; transform: rotate(270deg) translate(-10em,0); transform-origin: 0 0; -moz-transform: rotate(270deg) translate(-10em,0); -moz-transform-origin: 0 0; -webkit-transform: rotate(270deg) translate(-10em,0); -webkit-transform-origin: 0 0;";
                $content = $tag_open . 'style="' . $style . '">' . $arr[0] . $tag_close;


                break;

            case "F":
                $newColumnName = "";
                $newFinalColumnName = "";
                $remain = array();
                $striptags = false;
                $tag = "";

                # 'Q:mailto|Z|V:mail_first|support@example.com'
                $arr = explode("|", $columnValue);
                foreach ($arr as $value) {
                    $kv = explode(":", $value);
                    switch ($kv[0]) {
                        case "Q":
                            $newColumnName = $kv[1];
                            #if(!$newColumnName) throw new syntaxException ( "Missing a 'reserved column name' for parameter 'Q' in column 'F': $columnValue","",__FILE__,__LINE__,$this->fr_error);
                            if ($newColumnName == 'F')
                                throw new SyntaxReportException ("Not allowed: 'F' as 'reserved column name' for parameter 'Q': $columnValue",
                                    "", __FILE__, __LINE__, $this->fr_error);
                            break;
                        case "Z":
//                            $show = false;
                            break;
                        case "T":
                            $striptags = TRUE;
                            break;
                        case "X":
                            $tag = $kv[1];
                            if (!$tag)
                                throw new SyntaxReportException ("Missing the 'tag' parameter for 'X'):  $columnValue",
                                    "", __FILE__, __LINE__, $this->fr_error);
                            break;
                        case "V":
                            $newFinalColumnName = $kv[1];
                            if (!$newFinalColumnName)
                                throw new SyntaxReportException ("Missing the 'name' parameter for 'V':  $columnValue",
                                    "", __FILE__, __LINE__, $this->fr_error);
                            break;
                        case 'F':
                            throw new SyntaxReportException ("Qualifier 'F' is not allowed inside of a column with column name 'F':  $columnValue",
                                "", __FILE__, __LINE__, $this->fr_error);
                            break;
                        # Save every non 'F' qualifier for later usage
                        default:
                            $remain[] = $value;
                            break;
                    }
                }

                # Check for needed action
                #			if(!$newColumnName) throw new syntaxException ( "Missing parameter 'Q' in column 'F': $columnValue","",__FILE__,__LINE__,$this->fr_error);
                if ($newColumnName)
                    $columnName = $newColumnName;
                else
                    $columnName = "no column name defined";

                # reconstruct remaining parameters
                $arr = implode("|", $remain);
                # render
                $content = $this->renderColumn($columnIndex, $columnName, $arr, $full_level, $rowIndex, $flagOutput);

                if ($newFinalColumnName)
                    $columnName = $newFinalColumnName;

                # striptags
                if ($striptags) $content = strip_tags($content);
                # tag
                if ($tag) {
                    $arr = explode(" ", $tag);
                    $content = "<" . $tag . ">" . $content . "</" . $arr[0] . ">";
                }
                break;

            default :
                if ($flagControl) {
                    $flagOutput = false;
                } else {
                    $content .= $columnValue;
                }
                break;
        }

        // Always save column values, even if they are hidden.
        $this->variables->resultArray[$full_level . "."][$columnName] = ($content == '' && $flagControl) ? $columnValue : $content;

        return $content;
    }

    /**
     * Executes the Command in $cmd
     * RC: if RC==0 Returns Output, else 'RC - Output'
     *
     * @param    string $cmd : contains the Comand
     * @return   string  The content that is displayed on the website
     */
    private function myExec($cmd) {

        exec($cmd, $arr, $rc);

        $output = implode('<BR>', $arr);
        if ($rc != 0)
            $output = $rc . " - " . $output;

        return ($output);
    }

    /**
     * The main method of the PlugIn
     *
     * @param    string $content : The PlugIn content
     * @param    array $conf : The PlugIn configuration
     * @return    string The content that is displayed on the website
     */
    //Checkt ob der Beginn von Array2 gleich ist wie Array1
    // gibt true/false zurück

    /**
     * Renders PageX: convert position content to token content. Respect default values depending on PageX
     *
     * @param    string $columnName
     * @param    string $columnValue
     * @return string rendered link
     *
     * $columnValue:
     * -------------
     * [<page id|alias>[&param=value&...]] | [record id] | [text] | [tooltip] | [msgbox] | [class] | [target] | [render mode] | [create hash]
     *
     * param[0]: <page id|alias>[&param=value&...]
     * param[1]: record id
     * param[2]: text
     * param[3]: tooltip
     * param[4]: msgbox
     * param[5]: class
     * param[6]: target
     * param[7]: render mode
     * param[8]: create hash
     * @throws SyntaxReportException
     */
    private function doFixColPosPage($columnName, $columnValue) {

        $link = "";

        // Split definition
        $param = explode('|', $columnValue);
        if (count($param) > 9)
            throw new SyntaxReportException ("Too many parameter (max=9): $columnValue", "", __FILE__, __LINE__, $this->fr_error);

        // make first 'P' lowercase
        $columnName = 'p' . substr($columnName, 1);

        // -- Page --
        // Split PageId|PageAlias and  URL Params
        $tmparr = explode('&', $param[0], 2);

        $link .= $this->composeLinkPart('p', $tmparr[0]);            // -- PageID --
        $link .= $this->composeLinkPart('U', $tmparr[1]);            // -- URL Params --
        $link .= $this->composeLinkPart('i', $param[1]);                // -- record id --
        $link .= $this->composeLinkPart('t', $param[2]);                // -- Text --
        $link .= $this->composeLinkPart('o', $param[3]);                // -- tooltip --
        $link .= $this->composeLinkPart('q', $param[4], $this->page_control["msgbox"][$columnName]);                // -- msgbox
        $link .= $this->composeLinkPart('c', $param[5]);                // -- class --
        $link .= $this->composeLinkPart('g', $param[6]);                // -- target --
        $link .= $this->composeLinkPart('r', $param[7]);                // -- render mode --

        if (!$param[8]) $param[8] = $this->page_control["hash"][$columnName]; // if no hash behaviour defined, use default
        if ($param[8] == "h") $link .= "h|";

        if ($this->page_control["icon"][$columnName])
            $link .= $this->page_control["icon"][$columnName] . "|";

        return ($link);
    }

    /**
     * The main method of the PlugIn
     *
     * @param    string $content : The PlugIn content
     * @param    array $conf : The PlugIn configuration
     * @return   string The content that is displayed on the website
     */
    //Check ob arr1 nur 1 Feld mehr hat als arr2

    /**
     * If there is a value (or a defaultValue): compose it together with qualifier and delimiter.
     *
     * @param    string $qualifier
     * @param    string $value
     * @param    string $defaultValue
     *
     * @return    string        rendered link
     */
    private function composeLinkPart($qualifier, $value, $defaultValue = "") {

        if (!$value) $value = $defaultValue;

        if ($value)
            return ($qualifier . ":" . $value . "|");

        return '';
    }

    /**
     * Renders pageX: extract token and determine if any default value has be applied
     *
     * @param    string $columnName
     * @param    string $columnValue
     *
     * @return    string        rendered link
     */
    private function doPage($columnName, $columnValue) {
        $defaultQuestion = '';

        $param = explode('|', $columnValue);

        # get all defaultvalues, depending on the columnname
        $defaultImage = $this->page_control["icon"][$columnName];
        $defaultHash = $this->page_control["hash"][$columnName];
        # define defaultquestion only, if pagetype needs a question
        if ($this->page_control["msgbox"][$columnName]) $defaultQuestion = 'q:' . $this->page_control["msgbox"][$columnName];

        foreach ($param as $key) {
            switch (substr($key, 0, 1)) {
                case 'P':
                case 'E':
                case 'N':
                case 'D':
                case 'H':
                case 'I':
                case 'S':
                case 'B':
                case 'C':
                    $defaultImage = '';    // if any of the img token is given: no default
                    break;
                case 'h':
                    $defaultHash = '';    // if a hash definition is given: no default
                    break;
                case 'q':
                    $defaultQuestion = '';    // if a question is given: no default
                    break;
            }
        }

        $columnValue .= "|";

        // append defaulst
        if ($defaultImage) $columnValue .= $defaultImage . "|";
        if ($defaultHash) $columnValue .= $defaultHash . "|";
        if ($defaultQuestion) $columnValue .= $defaultQuestion . "|";

#debug($columnValue);

        return ($columnValue);
    }

    /**
     * Generate SortArgument
     *
     * @param $variable
     * @return string
     */
    private function getResultArrayIndex($variable) {

        $variable = substr($variable, 1, strlen($variable));
        return "[" . preg_replace_callback("/[a-z]/", "replaceToIndex", $variable) . "][" . preg_replace_callback("/[^a-z]/", "replaceToIndex", $variable) . "]";

    }

    /**
     * @param $arr1
     * @param $arr2
     * @return bool
     */
    private function compareArraystart($arr1, $arr2) {

        for ($i = 0; $i < count($arr1); $i++) {
            if ($arr1[$i] != $arr2[$i]) {
                return false;
            }
        }
        return true;
    }

    /**
     * @param $arr1
     * @param $arr2
     * @return bool
     */
    private function compareArraylength($arr1, $arr2) {

        if (count($arr1) + 1 == count($arr2)) {
            return true;
        }
        return false;
    }
}
